"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.saveSnapshot = exports.dumpConsole = exports.createBrowserHandlesForObject = exports.getPropertyDescriptor = exports.createBrowserHandle = exports.wrapPageFunction = void 0;
const tslib_1 = require("tslib");
const node_util_1 = require("node:util");
const log_1 = tslib_1.__importDefault(require("@apify/log"));
const apify_1 = require("apify");
const consts_1 = require("./consts");
const tools_1 = require("./tools");
/**
 * Creates a string with an appended pageFunction to be evaluated in
 * the browser context and placed within the given namespace.
 */
function wrapPageFunction(pageFunctionString, namespace) {
    return `if (typeof window['${namespace}'] !== 'object') window['${namespace}'] = {};
    window['${namespace}'].pageFunction = ${pageFunctionString}`;
}
exports.wrapPageFunction = wrapPageFunction;
/**
 * Attaches the provided function to the Browser context
 * by exposing it via page.exposeFunction. Returns a string
 * handle to be used to reference the exposed function in
 * the browser context.
 */
async function createBrowserHandle(page, fun) {
    const handle = await (0, tools_1.createRandomHash)();
    await page.exposeFunction(handle, fun);
    return handle;
}
exports.createBrowserHandle = createBrowserHandle;
/**
 * Looks up a property descriptor for the given key in
 * the given object and its prototype chain.
 */
function getPropertyDescriptor(target, key) {
    const descriptor = Reflect.getOwnPropertyDescriptor(target, key);
    if (descriptor) {
        return descriptor;
    }
    const prototype = Reflect.getPrototypeOf(target);
    if (prototype === Reflect.getPrototypeOf({}) || !prototype) {
        return null;
    }
    return getPropertyDescriptor(prototype, key);
}
exports.getPropertyDescriptor = getPropertyDescriptor;
/**
 * Exposes selected properties of an instance (of a Class or just an Object)
 * in the Browser context and returns their mapping.
 *
 * @param getters as TS will build all module methods as getters, we need to whitelist what are actual getters here
 */
async function createBrowserHandlesForObject(page, instance, properties, getters = []) {
    const promises = properties.map((prop) => {
        const descriptor = getPropertyDescriptor(instance, prop);
        if (!descriptor) {
            throw new Error(`Cannot create a browser handle for property: ${prop} on object ${instance}. No such property descriptor.`);
        }
        if (descriptor.value) {
            return {
                name: prop,
                value: descriptor.value,
                type: typeof descriptor.value === 'function' ? 'METHOD' : 'VALUE',
            };
        }
        if (descriptor.get) {
            const value = getters.includes(prop) ? descriptor.get : descriptor.get();
            const type = getters.includes(prop) ? 'GETTER' : 'METHOD';
            return { name: prop, value, type };
        }
        throw new Error(`Cannot create a browser handle for property: ${prop} on object ${instance}. No getter or value for descriptor.`);
    }).map(async ({ name, value, type }) => {
        if (/^METHOD|GETTER$/.test(type))
            value = await createBrowserHandle(page, value.bind(instance));
        return { name, value, type };
    });
    const props = await Promise.all(promises);
    return props.reduce((mappings, { name, value, type }) => {
        mappings[name] = { value, type };
        return mappings;
    }, {});
}
exports.createBrowserHandlesForObject = createBrowserHandlesForObject;
/**
 * Attaches a console listener to page's console that
 * mirrors all console messages to the Node context.
 *
 * This is used instead of the "dumpio" launch option
 * to prevent cluttering the STDOUT with unnecessary
 * Chromium messages, usually internal errors, occurring in page.
 */
function dumpConsole(page, options = {}) {
    page.on('console', async (msg) => {
        const msgType = msg.type();
        if (msgType === 'error' && !options.logErrors)
            return;
        // Do not ever log "Failed to load resource" errors, because they flood the log.
        if (msg.text() === consts_1.RESOURCE_LOAD_ERROR_MESSAGE)
            return;
        // Check for JSHandle tags in .text(), since .args() will
        // always include JSHandles, even for strings.
        const hasJSHandles = msg.text().includes('JSHandle@');
        // If there are any unresolved JSHandles, get their JSON representations.
        // Otherwise, just use the text immediately.
        let message;
        if (hasJSHandles) {
            const msgPromises = msg.args().map((jsh) => {
                return jsh.jsonValue()
                    .catch((e) => log_1.default.exception(e, `Stringification of console.${msgType} in browser failed.`));
            });
            message = (await Promise.all(msgPromises))
                .map((m) => (0, node_util_1.inspect)(m))
                .join(' '); // console.log('a', 'b') produces 'a b'
        }
        else {
            message = msg.text();
        }
        if (msgType in log_1.default) {
            log_1.default[msgType](message);
        }
        else {
            log_1.default.info(message);
        }
    });
}
exports.dumpConsole = dumpConsole;
/**
 * Tracking variable for snapshot throttling.
 */
let lastSnapshotTimestamp = 0;
/**
 * Saves raw body and a screenshot to the default key value store
 * under the SNAPSHOT-BODY and SNAPSHOT-SCREENSHOT keys.
 */
async function saveSnapshot({ page, body, contentType, json }) {
    // Throttle snapshots.
    const now = Date.now();
    if (now - lastSnapshotTimestamp < consts_1.SNAPSHOT.TIMEOUT_SECS * 1000) {
        log_1.default.warning('Aborting saveSnapshot(). It can only be invoked once '
            + `in ${consts_1.SNAPSHOT.TIMEOUT_SECS} secs to prevent database overloading.`);
        return;
    }
    lastSnapshotTimestamp = now;
    if (json) {
        await apify_1.Actor.setValue(consts_1.SNAPSHOT.KEYS.BODY, json);
    }
    else if (body && contentType) {
        await apify_1.Actor.setValue(consts_1.SNAPSHOT.KEYS.BODY, body, { contentType });
    }
    else if (page) {
        const htmlP = page.content();
        const screenshotP = page.screenshot();
        const [html, screenshot] = await Promise.all([htmlP, screenshotP]);
        await Promise.all([
            apify_1.Actor.setValue(consts_1.SNAPSHOT.KEYS.BODY, html, { contentType: 'text/html' }),
            apify_1.Actor.setValue(consts_1.SNAPSHOT.KEYS.SCREENSHOT, screenshot, { contentType: 'image/png' }),
        ]);
    }
    else {
        throw new Error('One of parameters "page" or "json" or "body" with "contentType" must be provided.');
    }
}
exports.saveSnapshot = saveSnapshot;
//# sourceMappingURL=browser_tools.js.map