"use strict";
var _a, _b;
Object.defineProperty(exports, "__esModule", { value: true });
exports.createContext = void 0;
const tslib_1 = require("tslib");
/* eslint-disable @typescript-eslint/no-unsafe-declaration-merging */
const log_1 = tslib_1.__importDefault(require("@apify/log"));
const apify_1 = require("apify");
const content_type_1 = tslib_1.__importDefault(require("content-type"));
const browser_tools_1 = require("./browser_tools");
const consts_1 = require("./consts");
const setup = Symbol('crawler-setup');
const internalState = Symbol('request-internal-state');
/**
 * Context represents everything that is available to the user
 * via Page Function. A class is used instead of a simple object
 * to avoid having to create new instances of functions with each
 * request.
 *
 * Some properties need to be accessible to the Context,
 * but should not be exposed to the user thus they are hidden
 * using a Symbol to prevent the user from easily accessing
 * and manipulating them.
 */
class Context {
    constructor(options) {
        Object.defineProperty(this, _a, {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, _b, {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "Actor", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: apify_1.Actor
        });
        Object.defineProperty(this, "Apify", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: apify_1.Actor
        }); // for back compatibility
        Object.defineProperty(this, "log", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: log_1.default
        });
        Object.defineProperty(this, "input", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "env", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "customData", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "globalStore", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        const { crawlerSetup, pageFunctionArguments, } = options;
        // Private
        this[setup] = crawlerSetup;
        this[internalState] = {
            skipLinks: false,
        };
        this.input = JSON.parse(crawlerSetup.rawInput);
        this.env = { ...crawlerSetup.env };
        this.customData = crawlerSetup.customData;
        this.globalStore = crawlerSetup.globalStore;
        // Page function arguments are directly passed from CrawlerSetup
        // and differ between Puppeteer and Cheerio Scrapers.
        // We must use properties and descriptors not to trigger getters / setters.
        Object.defineProperties(this, Object.getOwnPropertyDescriptors(pageFunctionArguments));
        // Bind this to allow destructuring off context in pageFunction.
        this.saveSnapshot = this.saveSnapshot.bind(this);
        this.skipLinks = this.skipLinks.bind(this);
        this.enqueueRequest = this.enqueueRequest.bind(this);
    }
    async getValue(...args) {
        return this[setup].keyValueStore.getValue(...args);
    }
    async setValue(...args) {
        return this[setup].keyValueStore.setValue(...args);
    }
    async saveSnapshot() {
        return (0, browser_tools_1.saveSnapshot)({
            page: this.page,
            body: this.body,
            contentType: this.contentType
                ? content_type_1.default.format(this.contentType)
                : null,
            json: this.json,
        });
    }
    skipLinks() {
        this[internalState].skipLinks = true;
    }
    async enqueueRequest(requestOpts = {}, options = {}) {
        const defaultRequestOpts = {
            useExtendedUniqueKey: true,
            keepUrlFragment: this.input.keepUrlFragments,
        };
        const newRequest = { ...defaultRequestOpts, ...requestOpts };
        const castedRequest = this.request;
        const defaultUserData = {
            [consts_1.META_KEY]: {
                parentRequestId: castedRequest.id || castedRequest.uniqueKey,
                depth: (castedRequest.userData?.[consts_1.META_KEY]).depth ?? 0 + 1,
            },
        };
        newRequest.userData = { ...defaultUserData, ...requestOpts.userData };
        return this[setup].requestQueue.addRequest(newRequest, options);
    }
}
_a = setup, _b = internalState;
/**
 * Creates a Context by passing all arguments to its constructor
 * and returns it, along with a reference to its state object.
 */
function createContext(contextOptions) {
    const context = new Context(contextOptions);
    return {
        context,
        state: context[internalState],
    };
}
exports.createContext = createContext;
//# sourceMappingURL=context.js.map