"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getMissingCookiesFromSession = exports.logPerformance = exports.createError = exports.maybeLoadPageFunctionFromDisk = exports.isPlainObject = exports.createRandomHash = exports.createDatasetPayload = exports.ensureMetaData = exports.checkInputOrThrow = exports.evalFunctionArrayOrThrow = exports.evalFunctionOrThrow = void 0;
const tslib_1 = require("tslib");
const node_crypto_1 = require("node:crypto");
const node_fs_1 = require("node:fs");
const node_path_1 = require("node:path");
const node_util_1 = require("node:util");
const node_vm_1 = require("node:vm");
const log_1 = tslib_1.__importDefault(require("@apify/log"));
const utils_1 = require("@crawlee/utils");
const ajv_1 = tslib_1.__importDefault(require("ajv"));
const consts_1 = require("./consts");
const randomBytes = (0, node_util_1.promisify)(node_crypto_1.randomBytes);
/**
 * Transforms a page function string into a Function object.
 * @param funcString
 */
function evalFunctionOrThrow(funcString) {
    let func;
    try {
        func = (0, node_vm_1.runInThisContext)(`(${funcString})`);
    }
    catch (err) {
        const e = err;
        throw new Error(`Compilation of pageFunction failed.\n${e.message}\n${e.stack.substr(e.stack.indexOf('\n'))}`);
    }
    if (typeof func !== 'function') {
        throw new Error('Input parameter "pageFunction" is not a function!');
    }
    return func;
}
exports.evalFunctionOrThrow = evalFunctionOrThrow;
/**
 * Transforms a pre/post navigation hooks string into array of Functions.
 * @param hooksString
 * @param paramName
 */
function evalFunctionArrayOrThrow(hooksString, paramName) {
    let arr;
    try {
        arr = (0, node_vm_1.runInThisContext)(`(${hooksString})`);
    }
    catch (err) {
        const e = err;
        throw new Error(`Compilation of ${paramName} failed.\n${e.message}\n${e.stack.substr(e.stack.indexOf('\n'))}`);
    }
    if (!Array.isArray(arr)) {
        throw new Error(`Input parameter "${paramName}" is not an array!`);
    }
    if (arr.some((func) => typeof func !== 'function')) {
        throw new Error(`Input parameter "${paramName}" is not an array of functions!`);
    }
    return arr;
}
exports.evalFunctionArrayOrThrow = evalFunctionArrayOrThrow;
/**
 * Validates the INPUT using the AJV library against the schema.
 */
function checkInputOrThrow(input, schema) {
    const ajv = new ajv_1.default({ allErrors: true, useDefaults: true, strict: false });
    const valid = ajv.validate(schema, input);
    if (!valid)
        throw new Error(`Invalid input:\n${JSON.stringify(ajv.errors, null, 2)}`);
}
exports.checkInputOrThrow = checkInputOrThrow;
/**
 * MODIFIES the provided Request by attaching necessary metadata.
 * Currently it only adds depth metadata, but it may be extended
 * as needed.
 */
function ensureMetaData(request) {
    request.userData ?? (request.userData = {});
    const metadata = request.userData[consts_1.META_KEY];
    if (!metadata) {
        request.userData[consts_1.META_KEY] = {
            depth: 0,
            parentRequestId: null,
        };
        return;
    }
    if (typeof metadata !== 'object') {
        throw new Error(`Request ${request.id} contains invalid metadata value.`);
    }
}
exports.ensureMetaData = ensureMetaData;
/**
 * Merges the result of the page function, that may be a single object
 * or an array objects, with request metadata and a flag, whether
 * an error occured. This would typically be used after the page
 * had been retried and the failedRequestHandler was called.
 *
 * If an Object[] is returned from the page function, each of the objects
 * will have the metadata appended for consistency, since the dataset
 * will flatten the results.
 */
function createDatasetPayload(request, response, pageFunctionResult, isError = false) {
    // Null and undefined do not prevent the payload
    // from being saved to dataset. It will just contain
    // the relevant metadata.
    let result = pageFunctionResult || {};
    // Validate the result.
    const type = typeof result;
    if (type !== 'object') {
        throw new Error(`Page function must return Object | Object[], but it returned ${type}.`);
    }
    // Metadata need to be appended to each item
    // to match results with dataset "lines".
    if (!Array.isArray(result))
        result = [result];
    const meta = {
        '#error': isError,
        '#debug': (0, utils_1.createRequestDebugInfo)(request, response),
    };
    return result.map((item) => ({ ...item, ...meta }));
}
exports.createDatasetPayload = createDatasetPayload;
/**
 * Creates a 12 byte random hash encoded as base64
 * to be used as identifier.
 */
async function createRandomHash() {
    return (await randomBytes(12))
        .toString('base64')
        .replace(/[+/=]/g, 'x') // Remove invalid chars.
        .replace(/^\d/, 'a'); // Ensure first char is not a digit.
}
exports.createRandomHash = createRandomHash;
/**
 * Checks whether an item is a plain object,
 * i.e. not a function or array as _.isObject()
 * would check for.
 */
function isPlainObject(item) {
    return (item && typeof item === 'object' && !Array.isArray(item));
}
exports.isPlainObject = isPlainObject;
/**
 * Attempts to load Page Function from disk if it's not available
 * on INPUT.
 */
function maybeLoadPageFunctionFromDisk(input, root) {
    if (input.pageFunction)
        return;
    const pageFunctionPath = (0, node_path_1.join)(root, consts_1.PAGE_FUNCTION_FILENAME);
    log_1.default.debug(`Loading Page Function from disk: ${pageFunctionPath}`);
    try {
        input.pageFunction = (0, node_fs_1.readFileSync)(pageFunctionPath, 'utf8');
    }
    catch (err) {
        log_1.default.exception(err, 'Page Function load from disk failed.');
    }
}
exports.maybeLoadPageFunctionFromDisk = maybeLoadPageFunctionFromDisk;
/**
 * Creates an error constructed using props
 * from the provided object.
 */
function createError(obj = {}) {
    const error = new Error(obj.message);
    error.stack = obj.stack;
    return error;
}
exports.createError = createError;
function logPerformance(request, title, hrtime) {
    if (log_1.default.getLevel() !== log_1.default.LEVELS.PERF)
        return;
    const runtime = process.hrtime(hrtime);
    const nanos = runtime[0] * 1000000000 + runtime[1];
    const micros = nanos / 1000;
    const millis = micros / 1000;
    log_1.default.perf(`${request.id} ${title} took ${Math.round(millis)} ms.`);
}
exports.logPerformance = logPerformance;
/**
 * Accepts an array of cookies in a { name, value }
 * format and finds if any of them are missing from
 * the session cookies for a given URL.
 */
function getMissingCookiesFromSession(session, cookies, url) {
    const sessionCookies = session.getCookies(url);
    return cookies.filter((c) => {
        const sessionHasCookie = sessionCookies.some((sc) => sc.name === c.name);
        return !sessionHasCookie;
    });
}
exports.getMissingCookiesFromSession = getMissingCookiesFromSession;
//# sourceMappingURL=tools.js.map